"use strict";

var _react = _interopRequireDefault(require("react"));

var _reactTestRenderer = _interopRequireDefault(require("react-test-renderer"));

var _utils = require("../utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

describe('parseBindingDefinition', function () {
  test('parse binding definitions correctly', function () {
    expect((0, _utils.parseBindingDefinition)('<')).toEqual({
      binding: '<',
      optional: false
    });
    expect((0, _utils.parseBindingDefinition)('&')).toEqual({
      binding: '&',
      optional: false
    }); // Optional

    expect((0, _utils.parseBindingDefinition)('<?')).toEqual({
      binding: '<',
      optional: true
    });
    expect((0, _utils.parseBindingDefinition)('&?')).toEqual({
      binding: '&',
      optional: true
    }); // Named

    expect((0, _utils.parseBindingDefinition)('<name')).toEqual({
      binding: '<',
      optional: false
    });
    expect((0, _utils.parseBindingDefinition)('&onClick')).toEqual({
      binding: '&',
      optional: false
    }); // Optional and named

    expect((0, _utils.parseBindingDefinition)('<?name')).toEqual({
      binding: '<',
      optional: true
    });
    expect((0, _utils.parseBindingDefinition)('&?onClick')).toEqual({
      binding: '&',
      optional: true
    });
  });
  test('throws an error for invalid bindings', function () {
    expect(function () {
      (0, _utils.parseBindingDefinition)(':');
    }).toThrowError("Defintion of binding ':' is not a valid");
  });
  test('throws an error for unsupported bindings', function () {
    expect(function () {
      (0, _utils.parseBindingDefinition)('@');
    }).toThrowError("Binding '@' is not a supported binding");
  });
});
describe('createMapScopeToProps', function () {
  test('throws errors for invalid or not supported bindings', function () {
    expect(function () {
      (0, _utils.createMapScopeToProps)({
        name: ':'
      });
    }).toThrowError("Defintion of binding ':' is not a valid");
    expect(function () {
      (0, _utils.createMapScopeToProps)({
        name: '@'
      });
    }).toThrowError("Binding '@' is not a supported binding");
  });
  test('create mapScopeToProps functions ', function () {
    const bindings = {
      name: '<',
      onClickName: '&'
    };
    const mapScopeToProps = (0, _utils.createMapScopeToProps)(bindings);
    expect(typeof mapScopeToProps).toBe('function');
  });
  test('mapScopeToProps returns correct props', function () {
    const bindings = {
      name: '<',
      onClickName: '&'
    };
    const scope = {
      name: 'Hello',
      onClickName: jest.fn(),
      unknownProp: 'UNKNOWN'
    };
    const event = Symbol();
    const mapScopeToProps = (0, _utils.createMapScopeToProps)(bindings);
    const props = mapScopeToProps(scope);
    expect(props.name).toBe(scope.name);
    expect(typeof props.onClickName).toBe('function');
    props.onClickName(event);
    expect(scope.onClickName.mock.calls[0]).toEqual([{
      $event: event
    }]);
  });
  test('mapScopeToProps handles optional bindings correctly', function () {
    const bindings = {
      name: '<?',
      onClickName: '&?',
      onDoubleClickName: '&?'
    };
    const scope = {
      name: 'Hello',
      onDoubleClickName: jest.fn()
    };
    const event = Symbol();
    const mapScopeToProps = (0, _utils.createMapScopeToProps)(bindings);
    const props = mapScopeToProps(scope);
    expect(props.name).toBe(scope.name);
    expect(typeof props.onClickName).toBe('undefined');
    expect(typeof props.onDoubleClickName).toBe('function');
    props.onDoubleClickName(event);
    expect(scope.onDoubleClickName.mock.calls[0]).toEqual([{
      $event: event
    }]);
  });
});
describe('wrapWithDecorators', function () {
  test('returns function', function () {
    const decorators = [];
    expect(typeof (0, _utils.wrapWithDecorators)()).toBe('function');
  });
  describe('renders correctly', function () {
    const decorators = [function (render) {
      return _react.default.createElement("span", {
        className: "Decorator-1"
      }, render());
    }, function (render) {
      return _react.default.createElement("span", {
        className: "Decorator-2"
      }, render());
    }];

    const ComponentToDecorate = function ({
      content
    }) {
      return _react.default.createElement("span", null, content);
    };

    const Component = (0, _utils.wrapWithDecorators)(decorators)(ComponentToDecorate);

    const tree = _reactTestRenderer.default.create(_react.default.createElement(Component, {
      content: "MyContent"
    })).toJSON();

    expect(tree).toMatchSnapshot();
  });
});